@@ -18,7 +18,8 @@ use stm32_hal2::{
     clocks::Clocks,
     dma::{self, Dma, DmaChannel, DmaInterrupt, DmaWriteBuf},
     gpio::{Pin, PinMode, Port},
-    low_power, pac,
+    low_power,
+    pac::{self, interrupt},
     prelude::*,
     spi::{self, BaudRate, Spi, SpiConfig, SpiDevice, SpiMode},
 };
@@ -74,8 +75,8 @@ fn main() -> ! {
     cs.set_low();
 
     unsafe {
-        spi.write_dma(&read_buf, DmaChannel::C3, &mut dma);
-        spi.read_dma(&mut read_buf, DmaChannel::C2, &mut dma);
+        spi.write_dma(&read_buf, DmaChannel::C3, Default::default(), &mut dma);
+        spi.read_dma(&mut read_buf, DmaChannel::C2, Default::default(), &mut dma);
     }
 
     while !dma.transfer_is_complete(DmaChannel::C2) {}
