@@ -60,6 +60,18 @@ pub enum StopWuck {
     Hsi = 1,
 }
 
+#[cfg(feature = "wb")]
+#[derive(Clone, Copy, PartialEq)]
+#[repr(u8)]
+/// RF system wakeup clock source selection
+pub enum RfWakeupSrc {
+    NoClock = 0b00,
+    /// LSE oscillator clock used as RF system wakeup clock
+    Lse = 0b01,
+    /// HSE oscillator clock divided by 1024 used as RF system wakeup clock
+    Hse = 0b11,
+}
+
 // L4 uses 0 - 4 only. Others use 1 - 15, but it's not clear when you'd
 // set more than WS5 or so.
 #[derive(Clone, Copy)]
@@ -481,15 +493,19 @@ pub struct Clocks {
     #[cfg(any(feature = "l4", feature = "l5", feature = "wb", feature = "wl"))]
     /// Select the input source to use after waking up from `stop` mode. Eg HSI or MSI.
     pub stop_wuck: StopWuck,
+    #[cfg(feature = "wb")]
+    /// Select the RF wakeup source.
+    pub rf_wakeup_src: RfWakeupSrc,
 }
 
 // todo: On L4/5, add a way to enable the MSI for use as CLK48.
 
 impl Clocks {
-    /// Setup common and return a `Valid` status if the config is valid. Return
+    /// Setup clocks and return a `Valid` status if the config is valid. Return
     /// `Invalid`, and don't setup if not.
     /// https://docs.rs/stm32f3xx-hal/0.5.0/stm32f3xx_hal/rcc/struct.CFGR.html
-    /// Use the STM32CubeIDE Clock Configuration tab to help.
+    /// Use the STM32CubeIDE Clock Configuration tab to help identify valid configs.
+    /// Use the `default()` implementation as a safe baseline.
     pub fn setup(&self, rcc: &mut RCC, flash: &mut FLASH) -> Result<(), SpeedError> {
         if let ClocksValid::NotValid = self.validate_speeds() {
             return Err(SpeedError {});
@@ -497,7 +513,7 @@ impl Clocks {
 
         // Adjust flash wait states according to the HCLK frequency.
         // We need to do this before enabling PLL, or it won't enable.
-        let (_, sysclk) = self.calc_sysclock();
+        let sysclk = self.sysclk();
 
         cfg_if! {
             if #[cfg(feature = "wb")] {
@@ -869,6 +885,10 @@ impl Clocks {
             }
         }
 
+        #[cfg(feature = "wb")]
+        rcc.csr
+            .modify(|_, w| unsafe { w.rfwkpsel().bits(self.rf_wakeup_src as u8) });
+
         // Enable and reset System Configuration Controller, ie for interrupts.
         // todo: Is this the right module to do this in?
         #[cfg(not(any(feature = "wb", feature = "wl")))] // todo: Do interrupts work without enabling syscfg on wb, which
@@ -1066,11 +1086,10 @@ impl Clocks {
     }
 
     /// Calculate the systick, and input frequency, in Hz.
-    fn calc_sysclock(&self) -> (u32, u32) {
-        let input_freq;
-        let sysclk = match self.input_src {
+    pub fn sysclk(&self) -> u32 {
+        match self.input_src {
             InputSrc::Pll(pll_src) => {
-                input_freq = match pll_src {
+                let input_freq = match pll_src {
                     #[cfg(not(any(feature = "g0", feature = "g4")))]
                     PllSrc::Msi(range) => range.value() as u32,
                     PllSrc::Hsi => 16_000_000,
@@ -1081,31 +1100,14 @@ impl Clocks {
             }
 
             #[cfg(not(any(feature = "g0", feature = "g4")))]
-            InputSrc::Msi(range) => {
-                input_freq = range.value() as u32;
-                input_freq
-            }
-            InputSrc::Hsi => {
-                input_freq = 16_000_000;
-                input_freq
-            }
-            InputSrc::Hse(freq) => {
-                input_freq = freq;
-                input_freq
-            }
+            InputSrc::Msi(range) => range.value() as u32,
+            InputSrc::Hsi => 16_000_000,
+            InputSrc::Hse(freq) => freq,
             #[cfg(feature = "g0")]
-            InputSrc::Lsi => {
-                input_freq = 32_000; // todo confirm this is right.
-                input_freq
-            }
+            InputSrc::Lsi => 32_000,
             #[cfg(feature = "g0")]
-            InputSrc::Lse => {
-                input_freq = 32_768;
-                input_freq
-            }
-        };
-
-        (input_freq, sysclk)
+            InputSrc::Lse => 32_768,
+        }
     }
 
     /// Check if the PLL is enabled. This is useful if checking wheather to re-enable the PLL
@@ -1120,11 +1122,6 @@ impl Clocks {
         rcc.cr.read().pllon().bit_is_set()
     }
 
-    pub fn sysclk(&self) -> u32 {
-        let (_, sysclk) = self.calc_sysclock();
-        sysclk
-    }
-
     pub fn hclk(&self) -> u32 {
         self.sysclk() / self.hclk_prescaler.value() as u32
     }
@@ -1195,8 +1192,6 @@ impl Clocks {
     }
 
     pub fn validate_speeds(&self) -> ClocksValid {
-        let mut result = ClocksValid::Valid;
-
         #[cfg(feature = "l4")]
         let max_clock = 80_000_000;
 
@@ -1215,6 +1210,9 @@ impl Clocks {
         #[cfg(feature = "wl")]
         let max_clock = 48_000_000;
 
+        // todo: Check valid PLL output range as well. You can use Cube, mousing over the PLL
+        // todo speed to find these.
+
         // todo: L4+ (ie R, S, P, Q) can go up to 120_000.
 
         #[cfg(any(feature = "l4", feature = "l5", feature = "wb"))]
@@ -1246,23 +1244,25 @@ impl Clocks {
         // todo: Note that this involves repeatedly calculating sysclk.
         // todo. We could work around thsi by calcing it once here.
         if self.sysclk() > max_clock {
-            result = ClocksValid::NotValid;
+            return ClocksValid::NotValid;
         }
 
+        // todo: What are the actual hclk limits? Not always sysclk?
+
         if self.hclk() > max_clock {
-            result = ClocksValid::NotValid;
+            return ClocksValid::NotValid;
         }
 
         if self.apb1() > max_clock {
-            result = ClocksValid::NotValid;
+            return ClocksValid::NotValid;
         }
 
         #[cfg(not(feature = "g0"))]
         if self.apb2() > max_clock {
-            result = ClocksValid::NotValid;
+            return ClocksValid::NotValid;
         }
 
-        result
+        ClocksValid::Valid
     }
 }
 
@@ -1316,6 +1316,8 @@ impl Default for Clocks {
             hsi48_on: false,
             #[cfg(any(feature = "l4", feature = "l5", feature = "wb", feature = "wl"))]
             stop_wuck: StopWuck::Msi,
+            #[cfg(feature = "wb")]
+            rf_wakeup_src: RfWakeupSrc::Lse,
         }
     }
 }
