@@ -3,9 +3,11 @@
 
 use crate::{
     clocks::{ClocksValid, SpeedError},
-    pac::{FLASH, RCC},
+    pac::{FLASH, PWR, RCC, SYSCFG},
 };
 
+use cfg_if::cfg_if;
+
 #[derive(Clone, Copy, PartialEq)]
 pub enum PllSrc {
     None,
@@ -20,9 +22,9 @@ impl PllSrc {
     /// See RCC_PLLCKSELR register, PLLSRC field.
     pub fn bits(&self) -> u8 {
         match self {
-            Self::Hse(_) => 0b00,
+            Self::Hsi(_) => 0b00,
             Self::Csi => 0b01,
-            Self::Hsi(_) => 0b10,
+            Self::Hse(_) => 0b10,
             Self::None => 0b11,
         }
     }
@@ -154,14 +156,104 @@ impl HsiDiv {
     }
 }
 
+#[derive(Clone, Copy, PartialEq)]
+#[repr(u8)]
+/// Range for the VSO. See H743 RM, section 6.8.6: PWR D3 domain control register. Sets PWR_D3CR,
+/// `VOS` field.
+pub enum VosRange {
+    /// 1.26 V - 1.40 V
+    #[cfg(not(feature = "h7b3"))]
+    VOS0 = 0, // This will actually be VOS1, but note that we handle the case of VOS0 activation
+    // differntly than the others, using its special activation sequence.
+    /// 1.15 V - 1.26 V
+    VOS1 = 0b11,
+    /// 1.05 V - 1.15 V
+    VOS2 = 0b10,
+    /// 0.95 V - 1.05 V
+    VOS3 = 0b01,
+}
+
+impl VosRange {
+    /// Power regulator voltage scale.
+    /// Choose the wait states based on VSO range and hclk frequency.. See H743 RM, Table 17: FLASH
+    /// recommended number of wait states and programming delay. Returns a tuple of (number of wait states,
+    /// programming delay) (FLASH ACR_LATENCY, WRHIGHFREQ) values respectively.
+    pub fn wait_states(&self, hclk: u32) -> (u8, u8) {
+        // todo: Feature gate for other H7 variants. This is likely only valid on H743 and/or
+        // todo 480MHz single-core variants.
+        match self {
+            #[cfg(not(feature = "h7b3"))]
+            Self::VOS0 => match hclk {
+                0..=70_000_000 => (0, 0),
+                70_000_001..=140_000_000 => (1, 1),
+                140_000_001..=185_000_000 => (2, 1),
+                185_000_001..=210_000_000 => (2, 2),
+                210_000_001..=225_000_000 => (3, 2),
+                225_000_001..=240_000_000 => (4, 2),
+                _ => panic!("Can't set higher than 240Mhz HCLK with VSO0 range."),
+            },
+            Self::VOS1 => match hclk {
+                0..=70_000_000 => (0, 0),
+                70_000_001..=140_000_000 => (1, 1),
+                140_000_001..=185_000_000 => (2, 1),
+                185_000_001..=210_000_000 => (2, 2),
+                210_000_001..=225_000_000 => (3, 2),
+                _ => panic!("Can't set higher than 225Mhz HCLK with VOS1 range."),
+            },
+            Self::VOS2 => match hclk {
+                0..=55_000_000 => (0, 0),
+                55_000_001..=110_000_000 => (1, 1),
+                110_000_001..=165_000_000 => (2, 1),
+                165_000_001..=225_000_000 => (3, 2),
+                _ => panic!("Can't set higher than 225Mhz HCLK with VSO2 range."),
+            },
+            Self::VOS3 => match hclk {
+                0..=45_000_000 => (0, 0),
+                45_000_001..=90_000_000 => (1, 1),
+                90_000_001..=135_000_000 => (2, 1),
+                135_000_001..=180_000_000 => (3, 2),
+                180_000_001..=225_000_000 => (4, 2),
+                _ => panic!("Can't set higher than 225Mhz HCLK with VSO3 range."),
+            },
+        }
+    }
+}
+
 /// Settings used to configure clocks.
 /// Note that we use integers instead of enums for some of the scalers, unlike in
 /// the other clock modules. This is due to the wide range available on these fields.
 pub struct Clocks {
     pub input_src: InputSrc,
+    /// Prescaler for PLL1
     pub divm1: u8,
+    /// PLL1 DIVN division factor
     pub divn1: u16,
+    /// PLL1 DIVP division factor
     pub divp1: u8,
+    /// PLL1 DIVQ division factor
+    pub divq1: u8,
+    /// PLL1 DIVR division factor
+    pub divr1: u8,
+    /// Prescaler for PLL2
+    pub divm2: u8,
+    /// PLL2 DIVN division factor
+    pub divn2: u16,
+    /// PLL2 DIVP division factor
+    pub divp2: u8,
+    /// PLL2 DIVQ division factor
+    pub divq2: u8,
+    /// PLL2 DIVR division factor
+    pub divr2: u8,
+    /// Prescaler for PLL3
+    pub divm3: u8,
+    /// PLL3 DIVN division factor
+    pub divn3: u16,
+    /// PLL3 DIVP division factor
+    pub divp3: u8,
+    /// PLL3 DIVQ division factor
+    pub divq3: u8,
+    /// PLL3 DIVR division factor
+    pub divr3: u8,
     pub d1_core_prescaler: HclkPrescaler,
     pub d1_prescaler: ApbPrescaler,
     pub hclk_prescaler: HclkPrescaler,
@@ -173,37 +265,98 @@ pub struct Clocks {
     pub hse_bypass: bool,
     pub security_system: bool,
     pub hsi48_on: bool,
+    pub vos_range: VosRange,
 }
 
 impl Clocks {
     /// Setup common and return a `Valid` status if the config is valid. Return
     /// `Invalid`, and don't setup if not.
     /// https://docs.rs/stm32f3xx-hal/0.5.0/stm32f3xx_hal/rcc/struct.CFGR.html
-    /// Use the STM32CubeIDE Clock Configuration tab to help.
-    pub fn setup(&self, rcc: &mut RCC, flash: &mut FLASH) -> Result<(), SpeedError> {
+    /// Use the STM32CubeIDE Clock Configuration tab to identify valid configs.
+    /// Use the `default()` implementation as a safe baseline.
+    /// This method also configures the PWR VOS setting, and can be used to enable VOS boost,
+    /// if `vos_range` is set to `VosRange::VOS0`.
+    pub fn setup(
+        &self,
+        rcc: &mut RCC,
+        flash: &mut FLASH,
+        pwr: &mut PWR,
+        syscfg: &mut SYSCFG,
+    ) -> Result<(), SpeedError> {
         if let ClocksValid::NotValid = self.validate_speeds() {
             return Err(SpeedError {});
         }
 
+        // Enable and reset System Configuration Controller, ie for interrupts.
+        // todo: Is this the right module to do this in?
+        rcc.apb4enr.modify(|_, w| w.syscfgen().set_bit());
+        rcc.apb4rstr.modify(|_, w| w.syscfgrst().set_bit());
+        rcc.apb4rstr.modify(|_, w| w.syscfgrst().clear_bit());
+
+        // H743 RM, sefction 6.8.6, and section 6.6.2: Voltage Scaling
+        //  Voltage scaling selection according to performance
+        // These bits control the VCORE voltage level and allow to obtains the best trade-off between
+        // power consumption and performance:
+        // – When increasing the performance, the voltage scaling shall be changed before increasing
+        // the system frequency.
+        // – When decreasing performance, the system frequency shall first be decreased before
+        // changing the voltage scaling.
+
+        match self.vos_range {
+            #[cfg(not(feature = "h7b3"))]
+            VosRange::VOS0 => {
+                // VOS0 activation/deactivation sequence
+                // The system maximum frequency can be reached by boosting the voltage scaling level to
+                // VOS0. This is done through the ODEN bit in the SYSCFG_PWRCR register.
+                // The sequence to activate the VOS0 is the following:
+                // 1. Ensure that the system voltage scaling is set to VOS1 by checking the VOS bits in
+                // PWR D3 domain control register (PWR D3 domain control register (PWR_D3CR))
+                pwr.d3cr
+                    .modify(|_, w| unsafe { w.vos().bits(VosRange::VOS1 as u8) });
+
+                // 2. Enable the SYSCFG clock in the RCC by setting the SYSCFGEN bit in the
+                // RCC_APB4ENR register.
+                rcc.apb4enr.modify(|_, w| w.syscfgen().set_bit());
+
+                // 3. Enable the ODEN bit in the SYSCFG_PWRCR register.
+                // PAC inconsistency between variants on if there's a modify field, and if
+                // `write` has a `bits()` or `bit()` method.
+                cfg_if! {
+                    if #[cfg(any(feature = "h747cm4", feature = "h747cm7"))] {
+                        syscfg.pwrcr.modify(|_, w| w.oden().set_bit());
+                    } else {
+                        syscfg.pwrcr.write(|w| unsafe { w.oden().bits(1) });
+                    }
+                }
+
+                // 4. Wait for VOSRDY to be set.
+                while pwr.d3cr.read().vosrdy().bit_is_clear() {}
+
+                // Once the VCORE supply has reached the required level, the system frequency can be
+                // increased. Figure 31 shows the recommended sequence for switching VCORE from VOS1 to
+                // VOS0 sequence.
+                // The sequence to deactivate the VOS0 is the following:
+                // 1. Ensure that the system frequency was decreased.
+                // 2. Ensure that the SYSCFG clock is enabled in the RCC by setting the SYSCFGEN bit set
+                // in the RCC_APB4ENR register.
+                // 3. Reset the ODEN bit in the SYSCFG_PWRCR register to disable VOS0.
+            }
+            _ => pwr
+                .d3cr
+                .modify(|_, w| unsafe { w.vos().bits(self.vos_range as u8) }),
+        }
+
         // Adjust flash wait states according to the HCLK frequency.
         // We need to do this before enabling PLL, or it won't enable.
-        let (_, sysclk) = self.calc_sysclock();
+        let sysclk = self.sysclk();
 
         let hclk = sysclk / self.hclk_prescaler.value() as u32;
-        // Reference manual section 3.3.3
+        // H742 RM, Table 17.
         // todo: What should this be?
+        let wait_states = self.vos_range.wait_states(hclk);
         flash.acr.modify(|_, w| unsafe {
-            if hclk <= 16_000_000 {
-                w.latency().bits(0b000)
-            } else if hclk <= 32_000_000 {
-                w.latency().bits(0b001)
-            } else if hclk <= 48_000_000 {
-                w.latency().bits(0b010)
-            } else if hclk <= 64_000_000 {
-                w.latency().bits(0b011)
-            } else {
-                w.latency().bits(0b100)
-            }
+            w.latency().bits(wait_states.0);
+            w.wrhighfreq().bits(wait_states.1)
         });
 
         // todo: Look up and document PLL config.
@@ -254,36 +407,6 @@ impl Clocks {
             w.hsebyp().bit(self.hse_bypass)
         });
 
-        if let InputSrc::Pll1(pll_src) = self.input_src {
-            // Turn off the PLL: Required for modifying some of the settings below.
-            rcc.cr.modify(|_, w| w.pll1on().clear_bit());
-            // Wait for the PLL to no longer be ready before executing certain writes.
-            while rcc.cr.read().pll1rdy().bit_is_set() {}
-
-            rcc.pllckselr.modify(|_, w| {
-                w.pllsrc().bits(pll_src.bits());
-                w.divm1().bits(self.divm1)
-            });
-
-            rcc.d1cfgr
-                .modify(|_, w| unsafe { w.hpre().bits(self.hclk_prescaler as u8) });
-
-            // rcc.pllcfgr.modify(|_, w| {
-            // });
-
-            rcc.pll1divr.modify(|_, w| unsafe {
-                w.divn1().bits(self.divn1);
-                w.divp1().bits(self.divp1)
-                // todo: divq1? r?
-            });
-
-            // Now turn PLL back on, once we're configured things that can only be set with it off.
-            // todo: Enable sai1 and 2 with separate settings, or lump in with mail PLL
-            // like this?
-            rcc.cr.modify(|_, w| w.pll1on().set_bit());
-            while rcc.cr.read().pll1rdy().bit_is_clear() {}
-        }
-
         rcc.cfgr
             .modify(|_, w| unsafe { w.sw().bits(self.input_src.bits()) });
 
@@ -301,6 +424,71 @@ impl Clocks {
         rcc.d3cfgr
             .modify(|_, w| unsafe { w.d3ppre().bits(self.d3_prescaler as u8) });
 
+        // Set USART2 to HSI, and USB to HSI48. Temp hardcoded.
+        #[cfg(not(feature = "h7b3"))]
+        rcc.d2ccip2r.modify(|_, w| unsafe {
+            w.usart234578sel().bits(0b111);
+            w.usbsel().bits(0b11)
+        });
+
+        if let InputSrc::Pll1(pll_src) = self.input_src {
+            // Turn off the PLL: Required for modifying some of the settings below.
+            rcc.cr.modify(|_, w| w.pll1on().clear_bit());
+            // Wait for the PLL to no longer be ready before executing certain writes.
+            while rcc.cr.read().pll1rdy().bit_is_set() {}
+
+            rcc.pllckselr.modify(|_, w| {
+                w.pllsrc().bits(pll_src.bits());
+                w.divm1().bits(self.divm1);
+                w.divm2().bits(self.divm2);
+                w.divm3().bits(self.divm3)
+            });
+
+            // Set and reset by software to select the proper reference frequency range used for PLL1.
+            // This bit must be written before enabling the PLL1.
+            let pll1_rng_val = match self.pll_input_speed(pll_src) {
+                1_000_000..=2_000_000 => 0b00,
+                2_000_001..=4_000_000 => 0b01,
+                4_000_001..=8_000_000 => 0b10,
+                8_000_001..=16_000_000 => 0b11,
+                _ => panic!("PLL1 input source must be between 1Mhz and 16Mhz."),
+            };
+
+            // todo: Use PLLCFGR to enable the various PLL1, 2, 3 R, Q, and P outputs as required
+            rcc.pllcfgr.modify(|_, w| w.pll1rge().bits(pll1_rng_val));
+
+            rcc.pll1divr.modify(|_, w| unsafe {
+                w.divn1().bits(self.divn1);
+                w.divp1().bits(self.divp1);
+                w.divq1().bits(self.divq1);
+                w.divr1().bits(self.divr1)
+            });
+
+            rcc.pll2divr.modify(|_, w| unsafe {
+                w.divn2().bits(self.divn2);
+                w.divp2().bits(self.divp2);
+                w.divq2().bits(self.divq2);
+                w.divr2().bits(self.divr2)
+            });
+
+            rcc.pll3divr.modify(|_, w| unsafe {
+                w.divn3().bits(self.divn3);
+                w.divp3().bits(self.divp3);
+                w.divq3().bits(self.divq3);
+                w.divr3().bits(self.divr3)
+            });
+
+            // Now turn PLL back on, once we're configured things that can only be set with it off.
+            // todo: Enable sai1 and 2 with separate settings, or lump in with mail PLL
+            // like this?
+            rcc.cr.modify(|_, w| w.pll1on().set_bit());
+            while rcc.cr.read().pll1rdy().bit_is_clear() {}
+
+            // todo: Enabling other PLLs?
+
+            // todo: SAI clock?
+        }
+
         rcc.cr.modify(|_, w| w.hsecsson().bit(self.security_system));
 
         if self.hsi48_on {
@@ -308,12 +496,6 @@ impl Clocks {
             while rcc.cr.read().hsi48rdy().bit_is_clear() {}
         }
 
-        // Enable and reset System Configuration Controller, ie for interrupts.
-        // todo: Is this the right module to do this in?
-        rcc.apb4enr.modify(|_, w| w.syscfgen().set_bit());
-        rcc.apb4rstr.modify(|_, w| w.syscfgrst().set_bit());
-        rcc.apb4rstr.modify(|_, w| w.syscfgrst().clear_bit());
-
         Ok(())
     }
 
@@ -379,35 +561,15 @@ impl Clocks {
         }
     }
 
-    /// Calculate the systick, and input frequency.
-    fn calc_sysclock(&self) -> (u32, u32) {
-        let input_freq;
-        let sysclk = match self.input_src {
-            InputSrc::Pll1(pll_src) => {
-                input_freq = match pll_src {
-                    PllSrc::Csi => 4_000_000,
-                    PllSrc::Hsi(div) => 64_000_000 / (div.value() as u32),
-                    PllSrc::Hse(freq) => freq,
-                    PllSrc::None => 0, // todo?
-                };
-                input_freq / self.divm1 as u32 * self.divn1 as u32 / self.divp1 as u32
-            }
-
-            InputSrc::Csi => {
-                input_freq = 4_000_000;
-                input_freq
-            }
-            InputSrc::Hsi(div) => {
-                input_freq = 64_000_000 / (div.value() as u32);
-                input_freq
-            }
-            InputSrc::Hse(freq) => {
-                input_freq = freq;
-                input_freq
-            }
+    /// Calculate the input speed to the PLL
+    fn pll_input_speed(&self, pll_src: PllSrc) -> u32 {
+        let input_freq = match pll_src {
+            PllSrc::Csi => 4_000_000,
+            PllSrc::Hsi(div) => 64_000_000 / (div.value() as u32),
+            PllSrc::Hse(freq) => freq,
+            PllSrc::None => 0,
         };
-
-        (input_freq, sysclk)
+        input_freq / self.divm1 as u32
     }
 
     /// Check if the PLL is enabled. This is useful if checking wheather to re-enable the PLL
@@ -423,12 +585,18 @@ impl Clocks {
     }
 
     pub fn sysclk(&self) -> u32 {
-        let (_, sysclk) = self.calc_sysclock();
-        sysclk
+        match self.input_src {
+            InputSrc::Pll1(pll_src) => {
+                self.pll_input_speed(pll_src) * self.divn1 as u32 / self.divp1 as u32
+            }
+            InputSrc::Csi => 4_000_000,
+            InputSrc::Hsi(div) => 64_000_000 / (div.value() as u32),
+            InputSrc::Hse(freq) => freq,
+        }
     }
 
     pub fn hclk(&self) -> u32 {
-        self.sysclk() / self.d1_core_prescaler as u32 / self.hclk_prescaler as u32
+        self.sysclk() / self.d1_core_prescaler.value() as u32 / self.hclk_prescaler.value() as u32
     }
 
     pub fn systick(&self) -> u32 {
@@ -470,57 +638,101 @@ impl Clocks {
         let mut result = ClocksValid::Valid;
 
         // todo: This depends on variant
-        let max_clock = 550_000_000;
+        // let max_clock = 550_000_000;
+        // #[cfg(feature = "h743")]
+        let max_sysclk = 480_000_000;
+        // #[cfg(feature = "h743")]
+        let max_hclk = 240_000_000;
+        // #[cfg(feature = "h743")]
+        let max_apb = 120_000_000; // todo: Different depending on apb
 
-        // todo: L4+ (ie R, S, P, Q) can go up to 120_000.
+        // todo: PLL input (Post DIVM1, pre DIVN1) must be between 1Mhz and 16 Mhz
+
+        // todo after pool today: FIgure out what other bit needs to be set to reflect this.
 
         // todo: Are these valid for all H7 configs?
-        if self.divm1 > 63 || self.divn1 > 512 || self.divp1 > 128 {
+        if self.divm1 > 63
+            || self.divm2 > 63
+            || self.divm3 > 63
+            || self.divn1 > 512
+            || self.divp1 > 128
+            || self.divp1 < 2
+        {
             return ClocksValid::NotValid;
         }
 
+        if let InputSrc::Pll1(pll_src) = self.input_src {
+            let pll_input_speed = self.pll_input_speed(pll_src);
+            if pll_input_speed < 1_000_000 || pll_input_speed > 16_000_000 {
+                return ClocksValid::NotValid;
+            }
+        }
+
+        // todo: More work on this, including feature gates
+
         // todo: QC these limits
         // todo: Note that this involves repeatedly calculating sysclk.
         // todo. We could work around thsi by calcing it once here.
-        if self.sysclk() > max_clock {
+        if self.sysclk() > max_sysclk {
             result = ClocksValid::NotValid;
         }
 
-        if self.hclk() > max_clock {
+        if self.hclk() > max_hclk {
             result = ClocksValid::NotValid;
         }
 
-        if self.apb1() > max_clock {
+        if self.apb1() > max_apb {
             result = ClocksValid::NotValid;
         }
 
-        if self.apb2() > max_clock {
+        if self.apb2() > max_apb {
             result = ClocksValid::NotValid;
         }
 
+        // todo: Apb3/4?
+
         result
     }
 }
 
 impl Default for Clocks {
-    /// This default configures common with a HSE, a 32Mhz sysclck. All peripheral common are at
-    /// 32 Mhz.
+    /// This default configures clocks with the HSI, and a 240Mhz sysclock speed.
+    /// Peripheral clocks are all set to 112Mhz, with timer clocks at 224Mhz.
     /// HSE output is not bypassed.
+    /// todo: Not 248Mhz due to an issue with wait states on Table 17 not showing higher than 225Mhz
+    /// // todo HCLK unless in VOS0.
     fn default() -> Self {
+        // todo: Feature-gate based on variant. Ie the 5xxMhz ones, the 240Mhz ones, and dual cores.
         Self {
             input_src: InputSrc::Pll1(PllSrc::Hsi(HsiDiv::Div1)),
-            divm1: 32,
-            divn1: 129,
-            divp1: 1,
+            divm1: 8,
+            // #[cfg(feature = "h743")]
+            // divn1: 240, // todo: For 480Mhz with right vco set.
+            // #[cfg(feature = "h743")]
+            divn1: 56,
+            divp1: 2,
+            divq1: 3, // Allows <150Mhz SAI clock, if it's configureud for PLL1Q.
+            divr1: 2,
+            divm2: 1,
+            divn2: 129,
+            divp2: 2,
+            divq2: 2,
+            divr2: 2,
+            divm3: 1,
+            divn3: 129,
+            divp3: 2,
+            divq3: 2,
+            divr3: 2,
             d1_core_prescaler: HclkPrescaler::Div1,
             d1_prescaler: ApbPrescaler::Div1,
             hclk_prescaler: HclkPrescaler::Div1,
-            d2_prescaler1: ApbPrescaler::Div1,
-            d2_prescaler2: ApbPrescaler::Div1,
-            d3_prescaler: ApbPrescaler::Div1,
+            d2_prescaler1: ApbPrescaler::Div2,
+            d2_prescaler2: ApbPrescaler::Div2,
+            d3_prescaler: ApbPrescaler::Div2,
             hse_bypass: false,
             security_system: false,
             hsi48_on: false,
+            vos_range: VosRange::VOS1,
         }
     }
 }
