@@ -1,4 +1,5 @@
-//! Inter-Integrated Circuit (I2C) bus. Also supports SMBUS. Provides APIs to configure, read, and write from
+//! Support for the Inter-Integrated Circuit (I2C) bus peripheral. Also supports SMBUS.
+//! Provides APIs to configure, read, and write from
 //! I2C, with blocking, nonblocking, and DMA functionality.
 
 use cast::{u16, u8};
@@ -15,19 +16,23 @@ use crate::{
     rcc_en_reset,
 };
 
-#[cfg(feature = "g0")]
+#[cfg(any(feature = "g0"))]
 use crate::pac::dma as dma_p;
 #[cfg(any(
     feature = "f3",
     feature = "l4",
     feature = "g4",
+    feature = "h7",
     feature = "wb",
     feature = "wl"
 ))]
 use crate::pac::dma1 as dma_p;
 
-#[cfg(not(any(feature = "h7", feature = "f4", feature = "l5")))]
-use crate::dma::{self, Dma, DmaChannel, DmaInput};
+#[cfg(not(any(feature = "f4", feature = "l5")))]
+use crate::dma::{self, ChannelCfg, Dma, DmaChannel};
+
+#[cfg(any(feature = "f3", feature = "l4"))]
+use crate::dma::DmaInput;
 
 macro_rules! busy_wait {
     ($regs:expr, $flag:ident, $variant:ident) => {
@@ -109,7 +114,8 @@ impl<R> I2c<R>
 where
     R: Deref<Target = pac::i2c1::RegisterBlock>,
 {
-    /// Configures the I2C peripheral. `freq` is in Hz. Doesn't check pin config.
+    /// Initialize a I2C peripheral, including configuration register writes, and enabling and resetting
+    /// its RCC peripheral clock. `freq` is in Hz.
     pub fn new(
         regs: R,
         device: I2cDevice,
@@ -243,7 +249,7 @@ where
         }
     }
 
-    /// Read multiple words.
+    /// Read multiple words to a buffer.
     pub fn read(&mut self, addr: u8, bytes: &mut [u8]) -> Result<(), Error> {
         // Wait for any previous address sequence to end
         // automatically. This could be up to 50% of a bus
@@ -383,7 +389,7 @@ where
         });
     }
 
-    #[cfg(not(any(feature = "g0", feature = "h7", feature = "f4", feature = "l5")))]
+    #[cfg(not(any(feature = "g0", feature = "f4", feature = "l5")))]
     /// Read data, using DMA. See L44 RM, 37.4.16: "Transmissino using DMA"
     /// Note that the `channel` argument is only used on F3 and L4.
     /// For a single write, set `autoend` to `true`. For a write_read and other use cases,
@@ -394,6 +400,7 @@ where
         buf: &[u8],
         autoend: bool,
         channel: DmaChannel,
+        channel_cfg: ChannelCfg,
         dma: &mut Dma<D>,
     ) where
         D: Deref<Target = dma_p::RegisterBlock>,
@@ -458,19 +465,24 @@ where
         // page 1169.
         // Note: If DMA is used for transmission, the TXIE bit does not need to be enabled
 
+        #[cfg(feature = "h7")]
+        let len = len as u32;
+        #[cfg(not(feature = "h7"))]
+        let len = len as u16;
+
         dma.cfg_channel(
             channel,
             &self.regs.txdr as *const _ as u32,
             ptr as u32,
-            len as u16,
+            len,
             dma::Direction::ReadFromMem,
             dma::DataSize::S8,
             dma::DataSize::S8,
-            Default::default(),
+            channel_cfg,
         );
     }
 
-    #[cfg(not(any(feature = "g0", feature = "h7", feature = "f4", feature = "l5")))]
+    #[cfg(not(any(feature = "g0", feature = "f4", feature = "l5")))]
     /// Read data, using DMA. See L44 RM, 37.4.16: "Reception using DMA"
     /// Note that the `channel` argument is only used on F3 and L4.
     pub unsafe fn read_dma<D>(
@@ -478,6 +490,7 @@ where
         addr: u8,
         buf: &mut [u8],
         channel: DmaChannel,
+        channel_cfg: ChannelCfg,
         dma: &mut Dma<D>,
     ) where
         D: Deref<Target = dma_p::RegisterBlock>,
@@ -525,15 +538,20 @@ where
         // SMBus Master receiver on page 1171.
         // Note: If DMA is used for reception, the RXIE bit does not need to be enabled
 
+        #[cfg(feature = "h7")]
+        let len = len as u32;
+        #[cfg(not(feature = "h7"))]
+        let len = len as u16;
+
         dma.cfg_channel(
             channel,
             &self.regs.rxdr as *const _ as u32,
             ptr as u32,
-            len as u16,
+            len,
             dma::Direction::ReadFromPeriph,
             dma::DataSize::S8,
             dma::DataSize::S8,
-            Default::default(),
+            channel_cfg,
         );
     }
 }
