@@ -23,7 +23,7 @@
 //!
 //! If you need `embedded-hal` traits, include the `embedded-hal` feature.
 //!
-//! You can review [this section of Cargo.toml](https://github.com/David-OConnor/stm32-hal/blob/main/Cargo.toml#L61)
+//! You can review [this section of Cargo.toml](https://github.com/David-OConnor/stm32-hal/blob/main/Cargo.toml#L82)
 //! to see which MCU and runtime features are available.
 //!
 //! ### Example highlights:
@@ -95,6 +95,8 @@
 // Some reg modifications are marked `unsafe` in some PAC crates, but not others.
 // Disable these warnings.
 #![allow(unused_unsafe)]
+// The `doc_cfg` feature allows us to show functionality that is feature-gated on `docs.rs`.
+#![feature(doc_cfg)]
 
 #[cfg(not(any(
     feature = "f301",
@@ -343,10 +345,10 @@ pub mod crc;
 // WB doesn't have a DAC. Some G0 variants do - add it! Most F4 variants have it, some don't
 pub mod dac;
 
-// todo: G0 missing many DMA registers like CCR? H7 DMA layout is different.
+// todo: G0 missing many DMA registers like CCR?
 // todo: F4 needs some mods. So, only working on L4 and G4.
 // todo: L5 has a PAC bug on CCR registers past 1.
-#[cfg(not(any(feature = "h7", feature = "f4", feature = "l5")))]
+#[cfg(not(any(feature = "f4", feature = "l5")))]
 pub mod dma;
 
 #[cfg(not(feature = "h7"))] // todo
@@ -354,6 +356,11 @@ pub mod flash;
 #[cfg(not(feature = "wl"))] // todo
 pub mod gpio;
 
+#[cfg(feature = "wb")]
+pub mod bluetooth;
+#[cfg(feature = "wb")]
+pub mod tl_mbox; // Mailbox for communicating with the RF core.
+
 #[cfg(feature = "wb")]
 pub mod hsem;
 
@@ -371,25 +378,34 @@ pub mod low_power;
 
 // F3, F4, L5, G0, and WL don't have Quad SPI.
 #[cfg(not(any(
-    feature = "f3",
-    feature = "f4",
-    feature = "l4x3", // todo: PAC bug?
-    feature = "l5",
-    feature = "g0",
-    feature = "g431",
-    feature = "g441",
-    feature = "g471",
-    feature = "g491",
-    feature = "g4a1",
-    feature = "h7b3",
-    feature = "wl",
+feature = "f3",
+feature = "f4",
+feature = "l4x3", // todo: PAC bug?
+feature = "l5",
+feature = "g0",
+feature = "g431",
+feature = "g441",
+feature = "g471",
+feature = "g491",
+feature = "g4a1",
+feature = "h7b3",
+feature = "wl",
 )))]
 pub mod qspi;
+
 #[cfg(not(feature = "wl"))] // todo
 pub mod rtc;
 
-// #[cfg(not(any(feature = "f3")))]
-// pub mod sai;
+// todo: Impl SAI on G4.
+#[cfg(not(any(
+    feature = "f3",
+    feature = "f4",
+    feature = "g0",
+    feature = "g4",
+    feature = "h7b3",
+    feature = "wl"
+)))]
+pub mod sai;
 
 pub mod spi;
 
@@ -466,7 +482,7 @@ macro_rules! make_simple_globals {
 // todo: Remove this debug_workaroudn function on MCUs that don't require it. Ie, is this required on G4? G0?
 use cortex_m::interrupt::free;
 
-#[cfg(not(any(feature = "g0", feature = "h7")))]
+#[cfg(not(any(feature = "g0")))]
 /// Workaround due to debugger disconnecting in WFI (and low-power) modes.
 /// This affects most (all?) STM32 devices. In production on battery-powered
 /// devices that don't use DMA, consider removing this, to prevent power
@@ -476,10 +492,18 @@ pub fn debug_workaround() {
     free(|_| {
         let dbgmcu = unsafe { &(*pac::DBGMCU::ptr()) };
 
-        #[cfg(not(feature = "l5"))]
-        dbgmcu.cr.modify(|_, w| w.dbg_sleep().set_bit());
-        dbgmcu.cr.modify(|_, w| w.dbg_stop().set_bit());
-        dbgmcu.cr.modify(|_, w| w.dbg_standby().set_bit());
+        cfg_if::cfg_if! {
+            if #[cfg(not(feature = "h7"))] {
+                #[cfg(not(feature = "l5"))]
+                dbgmcu.cr.modify(|_, w| w.dbg_sleep().set_bit());
+                dbgmcu.cr.modify(|_, w| w.dbg_stop().set_bit());
+                dbgmcu.cr.modify(|_, w| w.dbg_standby().set_bit());
+            } else {
+                dbgmcu.cr.modify(|_, w| w.dbgsleep_d1().set_bit());
+                dbgmcu.cr.modify(|_, w| w.dbgstop_d1().set_bit());
+                dbgmcu.cr.modify(|_, w| w.dbgstby_d1().set_bit());
+            }
+        }
     });
 
     free(|cs| {
