@@ -1,10 +1,11 @@
-//! Serial Peripheral Interface (SPI) bus. Provides APIs to configure, read, and write from
+//! Support for the Serial Peripheral Interface (SPI) bus peripheral.
+//! Provides APIs to configure, read, and write from
 //! SPI, with blocking, nonblocking, and DMA functionality.
 
 use core::{
     ops::Deref,
     ptr,
-    sync::atomic::{self, Ordering},
+    // sync::atomic::{self, Ordering},
 };
 
 use cortex_m::interrupt::free;
@@ -12,8 +13,6 @@ use cortex_m::interrupt::free;
 #[cfg(feature = "embedded-hal")]
 use embedded_hal::spi::FullDuplex;
 
-use nb::block;
-
 use crate::{
     pac::{self, RCC},
     rcc_en_reset,
@@ -25,13 +24,17 @@ use crate::pac::dma as dma_p;
     feature = "f3",
     feature = "l4",
     feature = "g4",
+    feature = "h7",
     feature = "wb",
     feature = "wl"
 ))]
 use crate::pac::dma1 as dma_p;
 
-#[cfg(not(any(feature = "h7", feature = "f4", feature = "l5")))]
-use crate::dma::{self, Dma, DmaChannel, DmaInput};
+#[cfg(not(any(feature = "f4", feature = "l5")))]
+use crate::dma::{self, Dma, DmaChannel};
+
+#[cfg(any(feature = "f3", feature = "l4"))]
+use crate::dma::DmaInput;
 
 use cfg_if::cfg_if;
 
@@ -215,7 +218,8 @@ impl<R> Spi<R>
 where
     R: Deref<Target = pac::spi1::RegisterBlock>,
 {
-    /// Configures the SPI peripheral to operate in full duplex master mode
+    /// Initialize an SPI peripheral, including configuration register writes, and enabling and resetting
+    /// its RCC peripheral clock.
     pub fn new(regs: R, device: SpiDevice, cfg: SpiConfig, baud_rate: BaudRate) -> Self {
         free(|_| {
             let rcc = unsafe { &(*RCC::ptr()) };
@@ -378,7 +382,7 @@ where
         Spi { regs, device, cfg }
     }
 
-    /// Change the baud rate of the SPI
+    /// Change the SPI baud rate.
     pub fn reclock(&mut self, baud_rate: BaudRate) {
         self.regs.cr1.modify(|_, w| w.spe().clear_bit());
 
