@@ -18,11 +18,20 @@ use cortex_m::interrupt::free;
 
 #[cfg(feature = "g0")]
 use crate::pac::dma as dma_p;
-#[cfg(any(feature = "f3", feature = "l4", feature = "g4", feature = "wb"))]
+#[cfg(any(
+    feature = "f3",
+    feature = "l4",
+    feature = "g4",
+    feature = "h7",
+    feature = "wb"
+))]
 use crate::pac::dma1 as dma_p;
 
-#[cfg(not(any(feature = "h7", feature = "f4", feature = "l5")))]
-use crate::dma::{self, Dma, DmaChannel, DmaInput};
+#[cfg(not(any(feature = "f4", feature = "l5")))]
+use crate::dma::{self, Dma, DmaChannel};
+
+#[cfg(any(feature = "f3", feature = "l4"))]
+use crate::dma::DmaInput;
 
 #[cfg(feature = "embedded-hal")]
 use embedded_hal::{
@@ -121,11 +130,12 @@ pub enum UsartInterrupt {
     TransmitEmpty,
 }
 
+/// Configuration for Usart. Can be used with default::Default.
 pub struct UsartConfig {
-    word_len: WordLen,
-    stop_bits: StopBits,
-    oversampling: OverSampling,
-    parity: Parity,
+    pub word_len: WordLen,
+    pub stop_bits: StopBits,
+    pub oversampling: OverSampling,
+    pub parity: Parity,
 }
 
 impl Default for UsartConfig {
@@ -151,6 +161,8 @@ impl<R> Usart<R>
 where
     R: Deref<Target = pac::usart1::RegisterBlock>,
 {
+    /// Initialize a U[s]ART peripheral, including configuration register writes, and enabling and
+    /// resetting its RCC peripheral clock. `baud` is the baud rate, in bytes-per-second.
     pub fn new(
         regs: R,
         device: UsartDevice,
@@ -310,7 +322,7 @@ where
         }
     }
 
-    /// Transmit data, as a sequence of u8.. See L44 RM, section 38.5.2: "Character transmission procedure"
+    /// Transmit data, as a sequence of u8. See L44 RM, section 38.5.2: "Character transmission procedure"
     pub fn write(&mut self, data: &[u8]) {
         // 7. Write the data to send in the USART_TDR register (this clears the TXE bit). Repeat this
         // for each data to be transmitted in case of single buffer.
@@ -483,7 +495,7 @@ where
         self.regs.cr3.modify(|_, w| w.dmat().set_bit());
     }
 
-    #[cfg(not(any(feature = "g0", feature = "h7", feature = "f4", feature = "l5")))]
+    #[cfg(not(any(feature = "g0", feature = "f4", feature = "l5")))]
     /// Receive data using DMA. (L44 RM, section 38.5.15)
     /// Note that the `channel` argument is only used on F3 and L4.
     pub unsafe fn read_dma<D>(&mut self, buf: &mut [u8], channel: DmaChannel, dma: &mut Dma<D>)
@@ -518,7 +530,10 @@ where
             UsartDevice::Three => dma.channel_select(DmaInput::Usart3Rx),
         }
 
-        // todo: Pri and Circular as args?
+        #[cfg(feature = "h7")]
+        let len = len as u32;
+        #[cfg(not(feature = "h7"))]
+        let len = len as u16;
 
         dma.cfg_channel(
             channel,
@@ -531,7 +546,7 @@ where
             // RXNE event.
             ptr as u32,
             // 3. Configure the total number of bytes to be transferred to the DMA control register.
-            len as u16,
+            len,
             dma::Direction::ReadFromPeriph,
             dma::DataSize::S8,
             dma::DataSize::S8,
